#include "RsVnaCustomDllInterface.h"
#include "string.h"
#include "stdio.h"

using namespace std;

RS_VNA_API int RsVnaOnLoadDll()
{
  return 0;
}

RS_VNA_API int RsVnaOnUnloadDll()
{
  return 0;
}

RS_VNA_API int RsVnaGetVersion(struct RsVnaVersion* dllVersion, struct RsVnaVersion* interfaceVersion)
{
  // return version of this dll
  dllVersion->majorRelease = 1;
  dllVersion->minorRelease = 0;
  dllVersion->patchLevel = 0;

  // we are following this interface specification
  interfaceVersion->majorRelease = RS_VNA_DLL_INTERFACE_MAJOR_RELEASE;
  interfaceVersion->minorRelease = RS_VNA_DLL_INTERFACE_MINOR_RELEASE;
  interfaceVersion->patchLevel = RS_VNA_DLL_INTERFACE_PATCH_LEVEL;

  return 0;
}

RS_VNA_API int RsVnaGetNumberOfTaskTypes(unsigned int* numberOfTaskTypes)
{
  *numberOfTaskTypes = 4;
  return 0;
}

RS_VNA_API int RsVnaGetTaskTypes(const unsigned int numberOfTaskTypes, char** taskTypesList, char** additionalDefaultInput)
{
  strncpy_s(taskTypesList[0], RS_VNA_MAX_CHAR_SIZE, "a1-Wave", _TRUNCATE);
  strncpy_s(taskTypesList[1], RS_VNA_MAX_CHAR_SIZE, "b1-Wave", _TRUNCATE);
  strncpy_s(taskTypesList[2], RS_VNA_MAX_CHAR_SIZE, "b1/a1-Ratio", _TRUNCATE);
  strncpy_s(taskTypesList[3], RS_VNA_MAX_CHAR_SIZE, "example device", _TRUNCATE);
  strncpy_s(*additionalDefaultInput, RS_VNA_MAX_CHAR_SIZE, "---", _TRUNCATE);
  return 0;
}

RS_VNA_API int RsVnaGetUnit(const struct RsVnaTaskId taskId, enum ERsVnaMeasUnit* unit)
{
  if (strcmp(taskId.taskType, "a1-Wave") == 0)
  {
    *unit = ERsVnaMeasUnit::RS_VNA_VOLT;
  }
  else if (strcmp(taskId.taskType, "b1-Wave") == 0)
  {
    *unit = ERsVnaMeasUnit::RS_VNA_VOLT;
  }
  else if (strcmp(taskId.taskType, "b1/a1-Ratio") == 0)
  {
    *unit = ERsVnaMeasUnit::RS_VNA_UNIT;
  }
  else if (strcmp(taskId.taskType, "example device") == 0)
  {
    *unit = ERsVnaMeasUnit::RS_VNA_AMPERE;
  }
  else
  {
    return 1;
  }
  return 0;
}

RS_VNA_API int RsVnaGetNumberOfHumanNames(const struct RsVnaTaskId taskId, unsigned int* numberOfHumanNames)
{
  if (strcmp(taskId.taskType, "a1-Wave") == 0)
  {
    *numberOfHumanNames = 1;
  }
  else if (strcmp(taskId.taskType, "b1-Wave") == 0)
  {
    *numberOfHumanNames = 1;
  }
  else if (strcmp(taskId.taskType, "b1/a1-Ratio") == 0)
  {
    *numberOfHumanNames = 2;
  }
  else if (strcmp(taskId.taskType, "example device") == 0)
  {
    *numberOfHumanNames = 0;
  }
  else
  {
    return 1;
  }
  return 0;
}

RS_VNA_API int RsVnaGetTraceRequirements(const struct RsVnaTaskId taskId,
  enum ERsVnaInputType* inputType,
  const unsigned int numberOfHumanNames,
  char** humanNamesList)
{
  char measString[5];
  if (strcmp(taskId.taskType, "a1-Wave") == 0)
  {
    *inputType = ERsVnaInputType::RS_VNA_LIST;
    sprintf(measString, "a1d%i", taskId.drivingPort);
    strncpy_s(humanNamesList[0], RS_VNA_MAX_CHAR_SIZE, measString, _TRUNCATE);
  }
  else if (strcmp(taskId.taskType, "b1-Wave") == 0)
  {
    *inputType = ERsVnaInputType::RS_VNA_LIST;
    sprintf(measString, "b1d%i", taskId.drivingPort);
    strncpy_s(humanNamesList[0], RS_VNA_MAX_CHAR_SIZE, measString, _TRUNCATE);
  }
  else if (strcmp(taskId.taskType, "b1/a1-Ratio") == 0)
  {
    *inputType = ERsVnaInputType::RS_VNA_LIST;
    sprintf(measString, "b1d%i", taskId.drivingPort);
    strncpy_s(humanNamesList[0], RS_VNA_MAX_CHAR_SIZE, measString, _TRUNCATE);
    sprintf(measString, "a1d%i", taskId.drivingPort);
    strncpy_s(humanNamesList[1], RS_VNA_MAX_CHAR_SIZE, measString, _TRUNCATE);
  }
  else if (strcmp(taskId.taskType, "example device") == 0)
  {
    *inputType = ERsVnaInputType::RS_VNA_SAME;
  }
  else
  {
    return 1;
  }
  return 0;
}

RS_VNA_API int RsVnaCalculateTrace(const struct RsVnaTaskId taskId,
  const unsigned int position,
  const unsigned int numberOfSweepPoints,
  const unsigned int numberOfTraceData,
  const struct RsVnaTraceData** traceDataList,
  struct RsVnaComplex* outputValue)
{
  char measString[5];
  if (strcmp(taskId.taskType, "a1-Wave") == 0)
  {
    *outputValue = traceDataList[0]->value;
  }
  else if (strcmp(taskId.taskType, "b1-Wave") == 0)
  {
    *outputValue = traceDataList[0]->value;
  }
  else if (strcmp(taskId.taskType, "b1/a1-Ratio") == 0)
  {
    double aR = traceDataList[0]->value.realPart;
    double aI = traceDataList[0]->value.imaginaryPart;
    double bR = traceDataList[1]->value.realPart;
    double bI = traceDataList[1]->value.imaginaryPart;

    double magSqAInv = 1.0 / (aR * aR + aI * aI);

    outputValue->realPart = (bR*aR + bI * aI) *magSqAInv;
    outputValue->imaginaryPart = (bI*aR - bR*aI) *magSqAInv;
  }
  else
  {
    return 1;
  }
  return 0;
}

RS_VNA_API int RsVnaGetInterruptRequests(const struct RsVnaTaskId taskId,
  enum ERsVnaInterruptPosition* interruptPositionForChannel,
  enum ERsVnaInterruptPosition* interruptPositionForSegment,
  enum ERsVnaInterruptPosition* interruptPositionForDrivingPort,
  enum ERsVnaInterruptPosition* interruptPositionForMeasPoint)
{
  *interruptPositionForMeasPoint = ERsVnaInterruptPosition::RS_VNA_POST;
  return 0;
}

RS_VNA_API int RsVnaInterruptCallback(const struct RsVnaTaskId taskId,
  const char* deviceAddress,
  const unsigned int position,
  const unsigned int numberOfSweepPoints,
  const enum ERsVnaInterruptType interruptType,
  const enum ERsVnaInterruptPosition interruptPosition,
  struct RsVnaComplex* outputValueList,
  unsigned int* positionOfLastOutputValue)
{
  char measString[5];
  if (strcmp(taskId.taskType, "example device") == 0)
  {
    outputValueList[position - 1].realPart = 1.0;
    outputValueList[position - 1].imaginaryPart = 1.0;
    *positionOfLastOutputValue = position;
  }
  else
  {
    return 1;
  }
  return 0;
}

RS_VNA_API int RsVnaGetErrorInfo(const int errorCode, char** errorText)
{
  switch (errorCode)
  {
  case 1:
    strncpy_s(*errorText, RS_VNA_MAX_CHAR_SIZE, "Wrong Task Name", _TRUNCATE);
    break;
  default:
    strncpy_s(*errorText, RS_VNA_MAX_CHAR_SIZE, "Unknown error code", _TRUNCATE);
    break;
  }

  return 0;
}
